/*
 * Lange Nacht der Wissenschaften 2012
 * 
 * Ansteuerung einer RGB-Leuchtdiode
 * 
 * 30.6.2012 Tschirley
 */

#include <stdbool.h>
#include <stdlib.h>
#include <msp430g2221.h>

/****************************************************/

#define LED_BIT   (BIT0|BIT1|BIT2)
#define LED_AUS	  !(LED_WEISS)
#define LED1	  BIT0
#define	LED2	  BIT1
#define	LED3	  BIT2
#define	LED_BLUE  BIT0
#define	LED_GREEN BIT1
#define	LED_RED	  BIT2
#define LED_WEISS BIT0|BIT1|BIT2
#define BLITZDAUER 1000
#define M_PAUSE   10000
#define M_LONG    20000
#define M_SHORT   10000
// Timer0 ticks
#define TICKS_PER_SEC (1000 / TACCR0)    // 1MHz clock

/****************************************************/
volatile char mode = 0;
static volatile unsigned long int sys_ticks = 0;
volatile unsigned char timerCounter = 0;
volatile unsigned int  intTimerCounter = 0;
volatile unsigned char whileCtr =0;
/****************************************************/
/* 
 * Funktionsprototypen
 */
void ConfigWDT( void );
void ConfigClocks( void );
void ConfigLEDs( void );

/*
 * Konfiguration des Watchdogs -- einfach ausschalten 
 */
void ConfigWDT( void )
{
  WDTCTL = WDTPW + WDTHOLD; // Stop WDT
}

/*
 * Konfigurieren der Takterzeugung
 */
void ConfigClocks( void )
{
  BCSCTL1 = CALBC1_1MHZ; 		    // Set range
  DCOCTL = CALDCO_1MHZ;  		    // Set DCO step + modulation 
  BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
  IFG1 &= ~OFIFG;                           // Clear OSCFault flag
  BCSCTL2 |= SELM_0 + DIVM_3 + DIVS_3;      // MCLK = DCO/8, SMCLK = DCO/8	
}

/*
 * Konfigurieren des Port1 mit den LEDs
 */
void ConfigPort1( void )
{
  P1DIR = 0;
  /* Initialisierung der LEDs                                   */ 
  P1DIR |= LED_BIT;     // Alle LED pins als Ausgang schalten
  P1DIR |= BIT6;	    // Rote LED auf LaunchPad
  P1OUT &= LED_BIT;  	// Alle LEDs ausschalten
  	
  P1IE  |= 0x08;        // P1.3 interrupt enabled
  P1IES |= 0x08;        // P1.3 Hi/lo edge
  P1IFG &= ~0x08;       // P1.3 IFG cleared
}


/*
 * Konfigurieren des Timers
 * Takt 1MHZ, CCR0 = 0x0100 liefert T = 256us
 */
void ConfigTimerA0( void )
{
  CCTL1 = OUTMOD_0;
  CCR1  = 32;
  CCTL0 = CCIE;
  CCR0  = 0x0100;
  TACTL = TASSEL_2 + MC_1; 	
}
/*
 * Timer Register neu einstellen um Grundfrequenz zu veraendern
 */
void ReconfigTimerA0_CCR0( unsigned long int regContent)
{
  CCR0 = regContent;
}

/****************************************************/
/*
 * Interrupt Service Routinen (ISR)
 */
/*
 * TIMER A0 ISR
 * 
 * Timer Interrupt, dieser wird immer aufgerufen, wenn der Timer ueberlaeuft. 
 * Hier passiert das zyklisch.
 * Aus dem Zaehlerstand von sys_ticks und dem Inhalt von led_brightness wird
 * die Ausgabe bestimmt. 
 */
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A_ISR( void )
{
  timerCounter++;
  intTimerCounter++;
}

/*
 * Port 1 interrupt service routine
 * 
 * Nach betaetigen der Taste wird der mode hochgezhlt. 
 */
#pragma vector=PORT1_VECTOR
__interrupt void Port_1_ISR( void )
{
  P1IFG &= ~0x08;             // P1.4 IFG cleared
  P1OUT = LED_AUS;	      // Alle LEDs Ausschalten
  mode++;	
  if( mode >= 0x10 )
    {
      mode = 0;
    }             
}

/****************************************************/
/* Relevante Funktionen fr den Programmablauf	    */

/*
 * Testfunktion zum Lampentest
 */
void LampTest( void )
{
  P1OUT = LED_RED;
  _delay_cycles( 10000 );
  P1OUT = LED_AUS;
  P1OUT = LED_BLUE;
  _delay_cycles( 10000 );
  P1OUT = LED_AUS;
  P1OUT = LED_GREEN;
  _delay_cycles( 10000 );
  P1OUT = LED_AUS;
  P1OUT = LED_BLUE|LED_GREEN;
  _delay_cycles( 30000 );
  P1OUT = LED_AUS;
  P1OUT = LED_WEISS;
  _delay_cycles( 50000 );
  P1OUT = LED_AUS;
  
  	
}
/*
 * Das ist das Hauptprogramm.
 * Was da genau passiert, steht in den Kommentraten im Quelltext.
 */
int main(void)
{
  /* Aufruf der Initialisierungsfunktion fuer den Controller     */
  ConfigWDT();		// Watchdog
  ConfigClocks();	// Takt
  ConfigPort1();	// LEDs
  ConfigTimerA0();	// Timer      	
  
  /* Einschalten aller Interrupts im Controller, die vorher     */
  /* initialisiert wurden                                       */
  _BIS_SR( GIE );

  /* Einmal eine Leuchtsequenz durchlaufen zum Selbsttest       */
  LampTest();
  /* Es startet eine Endlosschleife, die der Controller         */
  /* abarbeitet, bis ihm der Strom ausgeht.                     */
  while(1)
    { 
      whileCtr++;
      /*
       * Lightshow Programme nach Wert von mode auswaehlen 
       */
      switch( mode )
        {	
        case 0x00:
          P1OUT = LED_AUS;
          break;
        case 0x01:			// Blau
          P1OUT = LED_BLUE;
          break;
        case 0x02:			// Rot
          P1OUT = LED_RED;
          break;
        case 0x03:			// Gruen
          P1OUT = LED_GREEN;
          break;	
        case 0x04:			// Margenta Mischfarbe
          P1OUT = LED_RED|LED_BLUE;
          break;	
        case 0x05:
          P1OUT = LED_RED|LED_GREEN;
          break;
        case 0x06:
          P1OUT = LED_GREEN|LED_BLUE;
          break;
        case 0x07:
          P1OUT = LED_WEISS;
          break;
        case 0x08:
          /*
           * Die Variable timerCounter laeuft von 0 bis 255, abhaengig vom
           * Zaehlerstand werden die LEDs geschaltet. Der Zaehler kann via
           * ADC in der Geschwindigkeit beeinflusst werden.
           */
          if ( timerCounter == 0x00 )	{ P1OUT = LED_AUS;		}
          if ( timerCounter >= 0x01 )	{ P1OUT = LED_GREEN;		}
          if ( timerCounter == 0x40 )   { P1OUT = LED_AUS;		}
          if ( timerCounter >= 0x40 )	{ P1OUT = LED_RED;		}
          if ( timerCounter >= 0xA0 )   { P1OUT = LED_BLUE;		}
          if ( timerCounter == 0xF0 )   { P1OUT = LED_RED;      }
          break;
        case 0x09:
          /*
           * Anderer Ansatz - feste Verzoegerung in Taktzyklen
           * realisiert hier ein Blitzen mit verschiedenen Abstaenden
           */
          P1OUT = LED_RED;   _delay_cycles( BLITZDAUER);
          P1OUT = LED_AUS;   _delay_cycles( 20000 );
          P1OUT = LED_BLUE;  _delay_cycles( BLITZDAUER );
          P1OUT = LED_AUS;   _delay_cycles( 5000 );
          P1OUT = LED_RED;   _delay_cycles( BLITZDAUER );
          P1OUT = LED_AUS;   _delay_cycles( 5000 );
          P1OUT = LED_GREEN; _delay_cycles( BLITZDAUER );
          P1OUT = LED_AUS;   _delay_cycles( 50000 );
          P1OUT = LED_BLUE;  _delay_cycles( BLITZDAUER );
          P1OUT = LED_AUS;   _delay_cycles( 100000 );
          P1OUT = LED_BLUE;  _delay_cycles( BLITZDAUER );
          P1OUT = LED_AUS;   _delay_cycles( 200000 );
          P1OUT = LED_RED;   _delay_cycles( BLITZDAUER );
          P1OUT = LED_AUS;   _delay_cycles( 100000 );
          P1OUT = LED_WEISS; _delay_cycles( BLITZDAUER );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          break;
        case 0x0A:
          /* 
           * Ein Weisses Strobe 
           */
          P1OUT = LED_WEISS; _delay_cycles( 10000 );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          break;
        case 0x0B:
          /* 
           * Ein Blaues, krzeres Strobe 
           */
          P1OUT = LED_BLUE; _delay_cycles( 1000 );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          break;
        case 0x0C:
          /* 
           * Kurze Sequenz Rot-Blau-Grn
           */
          P1OUT = LED_RED;   _delay_cycles( 1000 );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          P1OUT = LED_BLUE;  _delay_cycles( 1000 );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          P1OUT = LED_GREEN; _delay_cycles( 1000 );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          break;
        case 0x0D:
          /* 
           * Blaue und Rote Doppelpulse
           */
          P1OUT = LED_BLUE; _delay_cycles( 3000 );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          P1OUT = LED_BLUE; _delay_cycles( 3000 );
          P1OUT = LED_AUS;   _delay_cycles( 20000 );
          P1OUT = LED_RED; _delay_cycles( 3000 );
          P1OUT = LED_AUS;   _delay_cycles( 10000 );
          P1OUT = LED_RED; _delay_cycles( 3000 );
          P1OUT = LED_AUS;   _delay_cycles( 20000 );
          break; 
        case 0x0E:
          /* 
           * Beuth als Morse 
           */
           /** B **/
          P1OUT = LED_WEISS; _delay_cycles( M_LONG  );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE+10000 );
          /** E **/
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE+10000 );
          /** U **/
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_LONG  );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE+10000 );
          /** T **/
          P1OUT = LED_WEISS; _delay_cycles( M_LONG  );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE+10000 );
          /** H **/
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE );
          P1OUT = LED_WEISS; _delay_cycles( M_SHORT );
          P1OUT = LED_AUS;   _delay_cycles( M_PAUSE+20000 );
          break;   
        default:
          P1OUT = LED_AUS;			// alles aus
          break;
        }
    }
}
